package tsp;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solution;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.util.ESat;

public class Tsp {
    public static void main(String[] args) {
        // les données du problèmes
        int n=5; // nombre de sommets
        int H=100; // borne sup. de la distance
        
        int[][] T; // matrice des distances
        T = new int[][] {
            {0, 1, 3, 5, 1},
            {1, 0, 4, 4, 3},
            {3, 4, 0, 4, 1},
            {5, 4, 4, 0, 3},
            {1, 3, 1, 3, 0}      
        };        
        
        Model mon_modele;
        mon_modele = new Model();
        
       // les variables         
	IntVar[] r;
	IntVar[] s;
	int[] coeff; // coefficients entiers pour les contraintes lineaires
	IntVar[] dp; // distance entre i et j
	IntVar d; // objectif


        // definition de r
        r = new IntVar[n];
        for (int i=0; i<n; i++){
          r[i] = mon_modele.intVar("r_"+i, 0,n-1);
        }        
        
        // definition de s
        s = new IntVar[n];
        for (int i=0; i<n; i++){
          s[i] = mon_modele.intVar("s_"+i, 0,n-1);
        }
        
        dp = new IntVar[n];
        for (int i=0; i<n; i++){
          dp[i] = mon_modele.intVar("dp_"+i, 0,H);
        }
                
        //s = mon_modele.intVarArray("s", n, 0, n-1); // domaine initial des variables succs
	d = mon_modele.intVar("d", 0, H*n);

        
        // contrainte numero 5.
        mon_modele.allDifferent(s).post();
        
        // contrainte numero 6.
        mon_modele.allDifferent(r).post();
        
        // contrainte numero 7.
        for (int i=0;i<n-1;i++){
 	 mon_modele.element(r[i+1], s, r[i], 0).post();
        }

        // contrainte numero 8.
        for (int i=0;i<n;i++){
	  mon_modele.element(dp[i], T[i], s[i]).post();
	}
        
        // fonction objectif
        mon_modele.sum(dp, "=", d).post();
      
        //for (int i=0; i<n; i++)
        //{
        //    IntVar[] colonne = new IntVar[n]; 
        //    int[] coeffs = new int[n];
        //    for (int j=0; j<n; j++){
        //            colonne[j] = dp[j];
        //            coeffs[j] = 1;
	    //}
        //    mon_modele.scalar(colonne, coeffs,"=",d).post();
        // }
        
        
          mon_modele.setObjective(Model.MINIMIZE, d);
        
          Solver mon_solveur= mon_modele.getSolver();
          Solution solution = new Solution(mon_modele);   
          while (mon_solveur.solve()){
            solution.record(); 
            System.out.println(d.toString());
            System.out.println("----------");
          }
        
         if (mon_solveur.isFeasible() == ESat.TRUE){
                System.out.println(solution.toString());
                int cout =(int) mon_solveur.getBestSolutionValue(); 
                System.out.println("Cmax = " + cout);
            }
            else
                System.out.println("pas de solution");
 }

 
    

    
}
